
#ifndef _NV_NERVJIT_H_
#define _NV_NERVJIT_H_

#include <llvm_common.h>

namespace nv
{

struct NervJITImpl;

class NVLLVM_EXPORT NervJIT
{
public:
    enum HeaderType {
        HEADER_SYSTEM,
        HEADER_ANGLED,
        HEADER_QUOTED,
    };

private:
    NervJITImpl* impl;

public:

    NervJIT();
    ~NervJIT();

    void loadModuleFromFiles(const FileList& files) {
        for(auto& f: files) {
            loadModuleFromFile(f);
        }
    }

    void clearMacroDefinitions();
    void addMacroDefinition(std::string val);
    void addMacroDefinition(const std::string& key, const std::string& val) {
        addMacroDefinition(key+"="+val);
    }
    
    void linkModule(const std::string& outFile, const std::vector<std::string>& inputList, bool onlyNeeded, bool internalize, bool optimize, bool preserveUseListOrder);
    
    void clearHeaderSearchPaths();
    void addHeaderSearchPath(std::string path, HeaderType htype);

    void generateBitcodeFromFile(const std::string& inputFile, std::string outFile);
    void generateBitcodeFromBuffer(const std::string& buffer, std::string outFile);
    
    void generatePCHFromFile(const std::string& inputFile, std::string outFile);
    void generatePCHFromBuffer(const std::string& buffer, std::string outFile);
    void usePCHFile(std::string pchfile);

    void loadModuleBitcode(const std::string& bcfile);

    void loadModuleFromFile(const std::string& file);

    void loadModuleFromBuffer(const std::string& buffer);
    
    uint64_t lookup(const std::string& name);

    void addCurrentProcess();
    void addDynamicLib(const char* filename);
    
    void setupCommandLine(const std::vector<std::string>& args);

};

};

#endif
